package com.xubx.springboot_01demo.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xubx.springboot_01demo.entity.dto.blog.AddBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.CollectBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.GetUserBlogsDto;
import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.entity.vo.HotBlog;
import com.xubx.springboot_01demo.service.BlogService;
import com.xubx.springboot_01demo.entity.vo.GetBlogDetailVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

@RestController //注解标识这是一个控制器类
@CrossOrigin    //加上CrossOrigin可解决跨域问题
@Slf4j
@Api(tags = "博客接口")
@RequestMapping("/blog")
public class BlogsController {
    @Resource
    BlogService blogService;

    /**
     * 获取所有博客
     */
    @GetMapping("/getBlogs")
    @ApiOperation("获取所有博客")
    public ResponseEntity<List<Blogs>> getBlogs() {
        log.info("获取所有博客");
        return ResponseEntity.ok(blogService.findAllBlogs());
    }

    /**
     * 获取博客详情
     *
     * @param blogId
     * @return
     */
    @GetMapping("/getBlogDetail")
    @ApiOperation("获取博客详情")
    public ResponseEntity<?> getBlogDetail(@RequestParam("blogId") int blogId) {
        log.info("获取博客详情，ID: {}", blogId);
        GetBlogDetailVo getBlogDetailVo = blogService.findByIdBlogs(blogId);

        if (getBlogDetailVo == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("博客不存在");
        }
        return ResponseEntity.ok(getBlogDetailVo);
    }

    /**
     * 浏览博客，增加阅读数
     *
     * @param blogId
     * @return {@link ResponseEntity }<{@link String }>
     */
    @GetMapping("/viewBlog")
    @ApiOperation("浏览博客")
    public ResponseEntity<String> viewBlog(@RequestParam("blogId") int blogId) {
        log.info("浏览博客，ID: {}", blogId);

        try {
            blogService.viewBlog(blogId);
            return ResponseEntity.ok("博客浏览成功");
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("博客浏览失败");
        }
    }

    /**
     * 当前用户点赞博客
     *
     * @param blogId
     * @return {@link ResponseEntity }<{@link String }>
     */
    @GetMapping("/likeBlog")
    @ApiOperation("点赞博客")
    public ResponseEntity<String> likeBlog(@RequestParam("blogId") int blogId) {
        log.info("点赞博客，ID: {}", blogId);

        try {
            blogService.likeBlog(blogId);
            return ResponseEntity.ok("接口调用成功");
        } catch (Exception e) {
            log.error("博客点赞失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("博客点赞失败");
        }
    }

    /**
     * 收藏博客
     *
     * @param collectBlogDto
     * @return {@link ResponseEntity }<{@link String }>
     */
    @PostMapping("/collectBlog")
    @ApiOperation("收藏博客")
    public ResponseEntity<String> collectBlog(@Valid @RequestBody CollectBlogDto collectBlogDto, BindingResult bindingResult) {
        log.info("收藏博客，ID: {}", collectBlogDto);

        if (bindingResult.hasErrors()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(bindingResult.getFieldError().getDefaultMessage());
        }

        try {
            blogService.collectBlog(collectBlogDto);
            return ResponseEntity.ok("接口调用成功");
        } catch (Exception e) {
            log.error("博客收藏失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("博客收藏失败");
        }
    }


    /**
     * 新增博客
     *
     * @param blogs
     */
    @PostMapping("/addBlog")
    @ApiOperation("新增博客")
    public ResponseEntity<String> addBlog(@Valid @RequestBody AddBlogDto blogs, BindingResult bindingResult) {
        log.info("新增博客入参：{}", JSONObject.toJSONString(blogs));

        if (bindingResult.hasErrors()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(bindingResult.getFieldError().getDefaultMessage());
        }
        try {
            blogService.addBlogs(blogs);
        } catch (Exception e) {
            log.error("博客新增失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("博客新增失败");
        }

        return ResponseEntity.status(HttpStatus.CREATED).body("博客新增成功");
    }

    /**
     * 修改博客
     *
     * @param blogs
     */
    @PostMapping("/updateBlog")
    @ApiOperation("修改博客")
    public ResponseEntity<String> updateBlog(@RequestBody Blogs blogs) {
        blogService.updateBlogs(blogs);
        return ResponseEntity.ok("博客更新成功");
    }

    /**
     * 删除博客
     *
     * @param id
     */
    @GetMapping("/deleteBlog")
    @ApiOperation("删除博客")
    public ResponseEntity<String> deleteBlog(@RequestParam("blogId") int id) {
        blogService.deleteBlogs(id);
        return ResponseEntity.ok("博客删除成功");
    }

    /**
     * 获取该用户的博客列表
     *
     * @param getUserBlogsDto
     * @param bindingResult
     * @return {@link ResponseEntity }<{@link Page }<{@link Blogs }>>
     */
    @PostMapping("/getUserBlogs")
    @ApiOperation("获取用户博客")
    public ResponseEntity<?> getUserBlogs(@Valid @RequestBody GetUserBlogsDto getUserBlogsDto, BindingResult bindingResult) {
        log.info("获取用户博客，{}", getUserBlogsDto);

        if (bindingResult.hasErrors()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(null);
        }

        try {
            Page<Blogs> blogs = blogService.findBlogsByUserId(getUserBlogsDto);
            return ResponseEntity.ok(blogs);
        } catch (Exception e) {
            log.error("获取用户博客失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("获取用户博客失败");
        }
    }

    /**
     * 兴趣推送
     * @return {@link ResponseEntity }<{@link List }<{@link Blogs }>>
     */
    @GetMapping("/pushBlogs")
    @ApiOperation("兴趣推送博客")
    public ResponseEntity<?> pushBlogs() {
        log.info("兴趣推送博客");

        try {
            List<Blogs> blogs = blogService.pushBlogs();
            return ResponseEntity.ok(blogs);
        } catch (Exception e) {
            log.error("兴趣推送博客失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("兴趣推送博客失败");
        }
    }

    /**
     * 获取热度排行榜
     * @return {@link List }<{@link HotBlog }>
     */
    @GetMapping("/listHotRank")
    public ResponseEntity<?> listHotRank() {
        log.info("获取热度排行榜");

        try{
            return ResponseEntity.ok(blogService.listHotRank());
        } catch (Exception e) {
            log.error("获取热度排行榜失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("获取热度排行榜失败");
        }
    }

    /**
     * 分类推送，根据分类推送博客
     * @param categoryId
     * @return {@link ResponseEntity }<{@link ? }>
     */
    @GetMapping("/pushBlogByCategory")
    public ResponseEntity<?> pushBlogByCategory(@RequestParam("categoryId") int categoryId) {
        log.info("根据分类推送博客，{}", categoryId);

        try {
            List<Blogs> blogs = blogService.pushBlogByCategory(categoryId);
            return ResponseEntity.ok(blogs);
        } catch (Exception e) {
            log.error("根据分类推送博客失败", e);
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("根据分类推送博客失败");
        }
    }


}
