package com.xubx.springboot_01demo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xubx.springboot_01demo.configuration.constantConfiguration;
import com.xubx.springboot_01demo.entity.dto.blog.AddBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.CollectBlogDto;
import com.xubx.springboot_01demo.entity.dto.blog.GetUserBlogsDto;
import com.xubx.springboot_01demo.entity.vo.HotBlog;
import com.xubx.springboot_01demo.mapper.BlogsMapper;
import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.entity.pojo.User;
import com.xubx.springboot_01demo.service.BlogService;
import com.xubx.springboot_01demo.service.InterestPushService;
import com.xubx.springboot_01demo.service.UserService;
import com.xubx.springboot_01demo.utils.token.RequestHolder;
import com.xubx.springboot_01demo.entity.vo.GetBlogDetailVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;
import java.util.stream.Collectors;

@Service
@Slf4j
public class BlogsServiceImpl extends ServiceImpl<BlogsMapper, Blogs> implements BlogService {
    @Resource
    BlogsMapper blogsMapper;

    @Autowired
    UserService userService;

    @Resource
    RedisTemplate redisTemplate;

    @Override
    public List<Blogs> findAllBlogs() {
        return blogsMapper.findAllBlogs();
    }

    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    InterestPushService interestPushService;


    /**
     * 获取博客的详情
     *
     * @param blogId
     * @return {@link GetBlogDetailVo }
     */
    @Override
    public GetBlogDetailVo findByIdBlogs(int blogId) {
        LambdaQueryWrapper<Blogs> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Blogs::getId, blogId);
        Blogs blog = blogsMapper.selectOne(queryWrapper);

        GetBlogDetailVo getBlogDetailVo = new GetBlogDetailVo();
        getBlogDetailVo.setId(blog.getId());
        getBlogDetailVo.setTitle(blog.getTitle());
        getBlogDetailVo.setContent(blog.getContent());
        getBlogDetailVo.setAuthorId(blog.getAuthorId());
        getBlogDetailVo.setCreatedTime(blog.getCreatedTime());
        getBlogDetailVo.setUpdatedTime(blog.getUpdatedTime());
        // 获取动态计数信息：点赞数、浏览数、收藏数、评论数
        //TODO 使用 Redis 分布式锁来确保只有一个线程能够从数据库中获取并更新缓存
        getBlogDetailVo.setLikeCount(getCachedCount(constantConfiguration.BLOG_LIKE_COUNT + blogId, () -> blogsMapper.getLikeCount(blogId)));
        getBlogDetailVo.setViewCount(getCachedCount(constantConfiguration.BLOG_VIEW_COUNT + blogId, () -> blogsMapper.getViewCount(blogId)));
        getBlogDetailVo.setCollectCount(getCachedCount(constantConfiguration.BLOG_COLLECT_COUNT + blogId, () -> blogsMapper.getCollectCount(blogId)));
        getBlogDetailVo.setCommentCount(getCachedCount(constantConfiguration.BLOG_COMMENT_COUNT + blogId, () -> blogsMapper.getCommentCount(blogId)));

        // 获取动态信息的状态：点赞状态，收藏状态
        getBlogDetailVo.setLikeStatus(getCachedSet(constantConfiguration.BLOG_LIKE_SET + blogId, () -> blogsMapper.getLikeStatus(blogId, RequestHolder.getuserId())));
        getBlogDetailVo.setCollectStatus(getCachedSet(constantConfiguration.BLOG_COLLECT_SET + blogId, () -> blogsMapper.getCollectStatus(blogId, RequestHolder.getuserId())));
        // 手动设置额外字段
        User author = userService.getUser(String.valueOf(blog.getAuthorId()));
        if (author != null) {
            getBlogDetailVo.setAuthorName(author.getUsername());
            getBlogDetailVo.setAvatar(author.getAvatar());
        }

        return getBlogDetailVo;
    }

    /**
     * 检查是否有该博客的动态计数，若无，则从数据库中查询并更新redis，若有，则直接从redis中获取
     *
     * @param key
     * @param dbQuery
     * @return int
     */
    private String getCachedCount(String key, Supplier<String> dbQuery) {
        String countExpiredKey = key + ":expired";    //逻辑过期的key
        // 尝试从 Redis 中获取计数
        String count = (String) redisTemplate.opsForValue().get(key);

        // 如果缓存中没有，则从数据库获取并写入缓存
        if (count == null) {
            count = dbQuery.get();
            if (count != null) {
                redisTemplate.opsForValue().set(key, count.toString());

                // 如果逻辑过期标志不存在，则创建并设置为 23 小时，确保缓存到期前同步
                if (!Boolean.TRUE.equals(redisTemplate.hasKey(countExpiredKey))) {
                    redisTemplate.opsForValue().set(countExpiredKey, "true", 23, TimeUnit.HOURS);
                }
            } else {
                count = "0"; // 如果数据库也没有数据，返回默认值 0
            }
        }

        return count;
    }

    /**
     * 检查redis的set集合中是否存在该用户，若无，将当前用户的点赞状态更新至redis
     *
     * @param key
     * @param dbQuery
     * @return {@link Integer }
     */
    private Integer getCachedSet(String key, Supplier<Integer> dbQuery) {
        String userId = RequestHolder.getuserId().toString();
        String blogId = key.split(":")[2];

        // 首先检查 Redis 中是否存在该 key
        Boolean keyExists = redisTemplate.hasKey(key);
        if (Boolean.FALSE.equals(keyExists)) {
            // 如果 Redis 中没有该 key，从数据库中获取点赞状态
            Integer status = dbQuery.get();

            // 获取该博客用户集合存入redis的set中
            List<String> userIds = blogsMapper.getUserIdsByBlogId(blogId);
            if (userIds != null && !userIds.isEmpty()) {
                redisTemplate.opsForSet().add(key, userIds.toArray());
            }
            return status;
        } else {
            // 如果 Redis 中存在 key，检查用户是否在 Set 中
            Boolean exists = redisTemplate.opsForSet().isMember(key, userId);
            return exists ? 1 : 0;
        }
    }

    @Override
    public void addBlogs(AddBlogDto blogDto) {
        LocalDateTime now = LocalDateTime.now();
        // 博客内容的插入
        Blogs blog = new Blogs();
        blog.setTitle(blogDto.getTitle());
        blog.setDescription(blogDto.getDescription());
        blog.setContent(blogDto.getContent());
        blog.setCoverImage(blogDto.getCoverImage());
        blog.setCategoryId(blogDto.getCategoryId());
        blog.setStatus(blogDto.getStatus());
        blog.setAuthorId(RequestHolder.getuserId());
        // TODO 后续改为AOP切面统一进行创建、更新时间的插入
        blog.setCreatedTime(Timestamp.valueOf(now));

        // 推入系统分类库，判断库中数据量是否达到1万条，如果达到了，进行分类id的自增
        Integer categoryId = blog.getCategoryId();
        String categoryFragmentationKey = constantConfiguration.CATEGORY_BLOGS_FRAGMENTATION + categoryId;
        final String categoryFragmentationNumber = (String) redisTemplate.opsForValue().get(categoryFragmentationKey);

        // 如果为null，则初始化为1
        if (categoryFragmentationNumber == null) {
            redisTemplate.opsForValue().set(categoryFragmentationKey, "1");
        }

        if (redisTemplate.opsForSet().size(constantConfiguration.CATEGORY_BLOGS + categoryId + ":" + categoryFragmentationNumber) >= 10000) {
            redisTemplate.opsForValue().increment(categoryFragmentationKey);
        }
        // 推入分类库
        interestPushService.pushCategoryStockIn(blog);

        // 冗余分片number字段
        blog.setCategoryFragmentation((String) redisTemplate.opsForValue().get(categoryFragmentationKey));
        blogsMapper.insert(blog);

        // 插入博客标签中间表
        blogDto.getTags().forEach(tag -> {
            blogsMapper.insertTags(blog.getId(), tag);
        });

        // 插入系统标签库
        interestPushService.pushTagsStockIn(blog.getId(), blogDto.getTags());

    }

    @Override
    public void updateBlogs(Blogs blogs) {
        blogsMapper.updateBlogs(blogs);
    }

    @Override
    public void deleteBlogs(int id) {
        blogsMapper.deleteBlogs(id);
        Blogs blog = blogsMapper.findByIdBlogs(id);
        //删除标签库和分类库中的博客
        interestPushService.deletTagsStockIn(blog);
        interestPushService.deletCategoryStockIn(blog);
    }

    /**
     * 根据用户id查询博客
     *
     * @param getUserBlogsDto
     * @return {@link List }<{@link Blogs }>
     */
    @Override
    public Page<Blogs> findBlogsByUserId(GetUserBlogsDto getUserBlogsDto) {
        User user = userService.getUser(getUserBlogsDto.getUserId());
        Page<Blogs> pageRequest = new Page<>(getUserBlogsDto.getPage(), getUserBlogsDto.getPageSize());

        LambdaQueryWrapper<Blogs> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Blogs::getAuthorId, user.getId())
                .orderByDesc(Blogs::getCreatedTime);
        return blogsMapper.selectPage(pageRequest, queryWrapper);

    }

    /**
     * 浏览博客增加博客阅读数，查看redis中是否有该key，若无则从数据库中获取
     *
     * @param blogId
     */
    @Override
    public void viewBlog(int blogId) {
        String userId = RequestHolder.getuserId().toString(); // 获取当前用户 ID

        // 定义 Redis 键
        String viewUserKey = constantConfiguration.BLOG_VIEW_SET + blogId; // 存放访问用户的 Set，带1小时过期
        String viewCountKey = constantConfiguration.BLOG_VIEW_COUNT + blogId; // 存放浏览数，带7天过期
        String viewCountExpireKey = constantConfiguration.BLOG_VIEW_COUNT + blogId + ":expired";// 逻辑过期标志

        // 检查用户是否在 1 小时内访问过该博客
        Boolean isFirstView = redisTemplate.opsForSet().isMember(viewUserKey, userId);

        if (isFirstView == null || !isFirstView) {
            // 如果用户未访问或 Redis key 不存在，增加浏览数
            redisTemplate.opsForSet().add(viewUserKey, userId);
            redisTemplate.expire(viewUserKey, 1, TimeUnit.HOURS);

            // 检查浏览数 key 是否存在
            String currentViewCount = (String) redisTemplate.opsForValue().get(viewCountKey);
            if (currentViewCount == null) {
                // 如果 Redis 中浏览数 key 不存在，从数据库获取并缓存
                currentViewCount = blogsMapper.getViewCount(blogId);
                redisTemplate.opsForValue().set(viewCountKey, currentViewCount);

                // 设置逻辑过期标志，确保缓存到期前会被同步到数据库
                if (!Boolean.TRUE.equals(redisTemplate.hasKey(viewCountExpireKey))) {
                    redisTemplate.opsForValue().set(viewCountExpireKey, "true", 23, TimeUnit.HOURS);
                }
            }

            redisTemplate.opsForValue().increment(viewCountKey);
            // 如果逻辑过期标志尚未存在，则创建并设置过期时间
            if (!Boolean.TRUE.equals(redisTemplate.hasKey(viewCountExpireKey))) {
                redisTemplate.opsForValue().set(viewCountExpireKey, "true", 23, TimeUnit.HOURS);
            }
        }
    }

    /**
     * 点赞博客增加博客点赞数
     *
     * @param blogId
     */
    @Override
    public void likeBlog(int blogId) {
        String userId = RequestHolder.getuserId().toString(); // 获取用户ID

        if (userId == null) {
            throw new IllegalStateException("用户未登录");
        }
        String likeUserKey = constantConfiguration.BLOG_LIKE_SET + blogId; // 用于存放点赞用户的 Set

        String likeCountKey = constantConfiguration.BLOG_LIKE_COUNT + blogId; // 用于存放点赞数
        String likeCountExpireKey = constantConfiguration.BLOG_LIKE_COUNT + blogId + ":expired"; // 逻辑过期标志
        String likeUserHashKey = constantConfiguration.BLOG_LIKE_HASH + blogId;// 用于存放点赞用户信息的 Hash
        String likeUserHashExpireKey = constantConfiguration.BLOG_LIKE_HASH + blogId + ":expired";

        // 检查是否已点赞，并确保没有发生空指针异常
        Boolean isFirstLike = redisTemplate.opsForSet().isMember(likeUserKey, userId);

        if (isFirstLike == null || !isFirstLike) {
            redisTemplate.opsForSet().add(likeUserKey, userId);
            redisTemplate.opsForValue().increment(likeCountKey);
            redisTemplate.opsForHash().put(likeUserHashKey, userId, "true");

            //更新用户模型
            interestPushService.updateUserModel(Integer.valueOf(userId), blogId, 1.0);

            // 设置过期时间仅在首次操作集合时
            if (!Boolean.TRUE.equals(redisTemplate.hasKey(likeCountExpireKey))) {
                redisTemplate.opsForValue().set(likeCountExpireKey, "true", 25, TimeUnit.HOURS);
            }
            if (!Boolean.TRUE.equals(redisTemplate.hasKey(likeUserHashExpireKey))) {
                redisTemplate.opsForValue().set(likeUserHashExpireKey, "true", 25, TimeUnit.HOURS);
            }
        } else {
            redisTemplate.opsForSet().remove(likeUserKey, userId);
            redisTemplate.opsForValue().decrement(likeCountKey);
            redisTemplate.opsForHash().put(likeUserHashKey, userId, "false");

            //更新用户模型
            interestPushService.updateUserModel(Integer.valueOf(userId), blogId, -1.0);

            // 设置逻辑过期标志，以确保下次同步到数据库
            if (!Boolean.TRUE.equals(redisTemplate.hasKey(likeCountExpireKey))) {
                redisTemplate.opsForValue().set(likeCountExpireKey, "true", 25, TimeUnit.HOURS);
            }
            if (!Boolean.TRUE.equals(redisTemplate.hasKey(likeUserHashExpireKey))) {
                redisTemplate.opsForValue().set(likeUserHashExpireKey, "true", 25, TimeUnit.HOURS);
            }
        }
    }

    /**
     * 收藏博客 增加博客收藏数 用户收藏夹 博客收藏中间表
     *
     * @param collectBlogDto
     */
    @Override
    public void collectBlog(CollectBlogDto collectBlogDto) {
        String userId = RequestHolder.getuserId().toString(); // 获取用户ID
        String blogId = collectBlogDto.getBlogId().toString();
        Integer favoritesId = collectBlogDto.getFavoriteId();
        if (userId == null) {
            throw new IllegalStateException("用户未登录");
        }
        String collectLockKey = constantConfiguration.BLOG_COLLECT_LOCK + blogId;
        // 尝试获取锁，设置锁的过期时间为10秒，防止死锁
        Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(collectLockKey, "LOCKED", 10, TimeUnit.SECONDS);

        if (Boolean.TRUE.equals(lockAcquired)) {
            try {
                String collectUserKey = constantConfiguration.BLOG_COLLECT_SET + blogId; // 用于存放收藏用户的 Set
                String collectCountKey = constantConfiguration.BLOG_COLLECT_COUNT + blogId; // 用于存放收藏数
                String collectCountExpireKey = constantConfiguration.BLOG_COLLECT_COUNT + blogId + ":expired"; // 用于存放收藏数
                String blogCollectBlogsHashKey = constantConfiguration.BLOG_COLLECT_HASH + favoritesId;
                String blogCollectBlogsHashExpiredKey = constantConfiguration.BLOG_COLLECT_HASH + favoritesId + ":expired";

                // 检查是否已收藏，并确保没有发生空指针异常
                Boolean isFirstCollect = redisTemplate.opsForSet().isMember(collectUserKey, userId);
//                Blogs blog = blogsMapper.selectById(collectBlogDto.getBlogId());
//                String blogJson = objectMapper.writeValueAsString(blog);

                if (isFirstCollect == null || !isFirstCollect) {
                    // 收藏
                    redisTemplate.opsForSet().add(collectUserKey, userId);
                    redisTemplate.opsForValue().increment(collectCountKey);

                    // 将收藏的博客缓存到用户收藏夹的博客列表中
                    redisTemplate.opsForHash().put(blogCollectBlogsHashKey, blogId, "true");

                    //更新用户模型
                    interestPushService.updateUserModel(Integer.valueOf(userId), collectBlogDto.getBlogId(), 2.0);

                    // 设置逻辑过期时间 确保能同步到数据中
                    if (!Boolean.TRUE.equals(redisTemplate.hasKey(collectCountExpireKey))) {
                        redisTemplate.opsForValue().set(collectCountExpireKey, "true", 23, TimeUnit.HOURS);
                    }
                    if (!Boolean.TRUE.equals(redisTemplate.hasKey(blogCollectBlogsHashExpiredKey))) {
                        redisTemplate.opsForValue().set(blogCollectBlogsHashExpiredKey, "true", 23, TimeUnit.HOURS);
                    }
                } else {
                    // 取消收藏
                    redisTemplate.opsForSet().remove(collectUserKey, userId);
                    redisTemplate.opsForValue().decrement(collectCountKey);
                    redisTemplate.opsForHash().put(blogCollectBlogsHashKey, blogId, "false");

                    //更新用户模型
                    interestPushService.updateUserModel(Integer.valueOf(userId), collectBlogDto.getBlogId(), -2.0);

                    // 设置逻辑过期时间 确保能同步到数据中
                    if (!Boolean.TRUE.equals(redisTemplate.hasKey(collectCountExpireKey))) {
                        redisTemplate.opsForValue().set(collectCountExpireKey, "true", 23, TimeUnit.HOURS);
                    }
                    if (!Boolean.TRUE.equals(redisTemplate.hasKey(blogCollectBlogsHashExpiredKey))) {
                        redisTemplate.opsForValue().set(blogCollectBlogsHashExpiredKey, "true", 23, TimeUnit.HOURS);
                    }
                }
            } finally {
                // 释放锁
                redisTemplate.delete(collectLockKey);
            }
        } else {
            log.warn("无法获取分布式锁，用户 {} 收藏博客 {} 操作被跳过", userId, blogId);
        }
    }

    /**
     * 兴趣推送博客
     *
     * @return {@link List }<{@link Blogs }>
     */
    @Override
    public List<Blogs> pushBlogs() {
        Integer userId = RequestHolder.getuserId();
        User user = null;
        if (userId != null) {
            user = userService.getUser(String.valueOf(userId));
        }
        Collection<String> blogIds = interestPushService.listBlogsByUserModel(user);
        List<Blogs> blogs = new ArrayList<>();

        // 如果该用户没有兴趣模型，默认给最新的十条博客
        if (ObjectUtils.isEmpty(blogIds)) {
            blogIds = list(new LambdaQueryWrapper<Blogs>().orderByDesc(Blogs::getCreatedTime)).stream().map(Blogs::getId).map(String::valueOf).collect(Collectors.toList());
            blogIds = new HashSet<>(blogIds).stream().limit(10).collect(Collectors.toList());
        }
        blogs = listByIds(blogIds);
        return blogs;
    }

    /**
     * 获取热度排行榜
     *
     * @return {@link List }<{@link HotBlog }>
     */
    @Override
    public List<HotBlog> listHotRank() {
        final Set<ZSetOperations.TypedTuple<Object>> zSet = redisTemplate.opsForZSet().reverseRangeWithScores(constantConfiguration.HOT_RANK, 0, -1);
        final List<HotBlog> hotBlogs = new ArrayList<>();

        for (ZSetOperations.TypedTuple<Object> objectTypedTuple : zSet) {
            final HotBlog hotBlog;
            try {
                hotBlog = objectMapper.readValue(objectTypedTuple.getValue().toString(), HotBlog.class);
                hotBlog.setHot(objectTypedTuple.getScore());
                hotBlog.formatHot();
                hotBlogs.add(hotBlog);
            } catch (JsonProcessingException e) {
                log.error("解析热度排行榜失败", e);
            }
        }
        return hotBlogs;
    }

    /**
     * 分类推送，根据分类推送博客
     * @param categoryId
     * @return {@link List }<{@link Blogs }>
     */
    @Override
    public List<Blogs> pushBlogByCategory(Integer categoryId) {
        if (categoryId == null) {
            throw new IllegalArgumentException("分类id不能为空");
        }
        Collection<String> blogIds = interestPushService.listBlogsByCategoryId(categoryId);
        List<Blogs> blogs;

        if (ObjectUtils.isEmpty(blogIds)) {
            throw new IllegalArgumentException("该分类下没有博客");
        }
        blogs = listByIds(blogIds);
        return blogs;
    }
}
