package com.xubx.springboot_01demo.service.impl;

import com.xubx.springboot_01demo.configuration.constantConfiguration;
import com.xubx.springboot_01demo.entity.pojo.Blogs;
import com.xubx.springboot_01demo.mapper.BlogsMapper;
import com.xubx.springboot_01demo.entity.pojo.User;
import com.xubx.springboot_01demo.service.InterestPushService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 兴趣推送
 *
 * @author Xubx
 * @date 2024/11/22
 */
@Service
public class InterestPushServiceImpl implements InterestPushService {

    @Autowired
    private RedisTemplate redisTemplate;
    @Resource
    private BlogsMapper blogsMapper;

    /**
     * 初始化用户模型
     *
     * @param userId
     * @param tags
     */
    @Override
    @Async("asyncExecutor")
    public void initUserModel(Integer userId, List<String> tags) {
        String userModelKey = constantConfiguration.USER_MODEL + userId;
        Map<Object, Object> modelMap = new HashMap<>();
        if (!ObjectUtils.isEmpty(tags)) {
            int size = tags.size();
            // 将标签分为等分概率，
            double probabilityValue = 100 / size;
            for (String tag : tags) {
                modelMap.put(tag, String.valueOf(probabilityValue));
            }
        }
        redisTemplate.delete(userModelKey);
        redisTemplate.opsForHash().putAll(userModelKey, modelMap);
    }

    /**
     * 更新用户模型
     * 若标签已存在：调整权重。
     * 若标签不存在：新增标签。
     * 若权重更新后小于等于 0：移除标签，避免无效标签影响推荐。
     */
    @Override
    @Async("asyncExecutor")
    public void updateUserModel(Integer userId, Integer blogId, Double score) {
        if (userId != null) {
            String userModelKey = constantConfiguration.USER_MODEL + userId;
            Map<Object, Object> modelMap = redisTemplate.opsForHash().entries(userModelKey);

            if (ObjectUtils.isEmpty(modelMap)) {
                modelMap = new HashMap<>();
            }

            // 获取博客下的所有标签
            List<String> tags = blogsMapper.getTagsByBlogId(blogId);
            if (!ObjectUtils.isEmpty(tags)) {
                for (String tag : tags) {
                    if (modelMap.containsKey(tag)) {
                        double newScore = Double.parseDouble((String) modelMap.get(tag)) + score;
                        if (newScore > 0) {
                            modelMap.put(tag, String.valueOf(newScore));
                        } else {
                            modelMap.remove(tag);
                        }
                    } else {
                        modelMap.put(tag, String.valueOf(score));
                    }
                }
            }
            // 平滑权重
            final int tagsSize = modelMap.keySet().size();
            for (Object tag : modelMap.keySet()) {
                double probabilityValue = (Double.parseDouble((String) modelMap.get(tag)) + tagsSize) / tagsSize;
                modelMap.put(tag, String.valueOf(probabilityValue));
            }

            redisTemplate.opsForHash().putAll(userModelKey, modelMap);
        }
    }

    /**
     * 兴趣推送
     *
     * @param user
     * @return {@link List }<{@link String }>
     */
    @Override
    public Collection<String> listBlogsByUserModel(User user) {
        // 非游客
        if (!ObjectUtils.isEmpty(user)) {
            String userModelKey = constantConfiguration.USER_MODEL + user.getId();
            final Map<Object, Object> modelMap = redisTemplate.opsForHash().entries(userModelKey);
            if (!ObjectUtils.isEmpty(modelMap)) {
                // 构造概率数组
                final String[] probabilityArray = initProbabilityArray(modelMap);

                // 随机选取x个标签
                final int x = 10;
                final Random randomObject = new Random();
                final List<String> tags = new ArrayList<>();
                for (int i = 0; i < x; i++) {
                    tags.add(probabilityArray[randomObject.nextInt(probabilityArray.length)]);
                }
                //随机获取redis中该标签下的博客id
                List<Object> list = redisTemplate.executePipelined((RedisCallback<Object>) connection -> {
                    for (String tag : tags) {
                        String key = constantConfiguration.TAG_BLOGS + tag;
                        connection.sRandMember(key.getBytes());
                    }
                    return null;
                });
                //TODO 根据观看历史进行已观看视频的去重
                return list.stream().filter(Objects::nonNull).map(Object::toString).collect(Collectors.toSet());
            }
        }
        //TODO 游客 随机获取十个标签
        return null;
    }

    /**
     * 新增博客推入标签库中
     *
     * @param bligId
     * @param tags
     */
    @Override
    @Async("asyncExecutor")
    public void pushTagsStockIn(Integer bligId, List<String> tags) {
        //使用pipelined批量执行
        redisTemplate.executePipelined((RedisCallback<Object>) connection -> {
            for (String tag : tags) {
                connection.sAdd((constantConfiguration.TAG_BLOGS + tag).getBytes(), String.valueOf(bligId).getBytes());
            }
            return null;
        });
    }

    /**
     * 新增博客推入分类库
     * @param blog
     */
    @Override
    @Async("asyncExecutor")
    public void pushCategoryStockIn(Blogs blog) {
        final String categoryId = String.valueOf(blog.getCategoryId());

        // 通过分片解决redis的大key问题
        final String categoryFragmentationNumber = (String) redisTemplate.opsForValue().get(constantConfiguration.CATEGORY_BLOGS_FRAGMENTATION + categoryId);

        redisTemplate.opsForSet().add(constantConfiguration.CATEGORY_BLOGS + categoryId + ":" + categoryFragmentationNumber, String.valueOf(blog.getId()));
    }

    /**
     * 分类推送，根据分类随机推送视频
     * @param categoryId
     * @return {@link Collection }<{@link String }>
     */
    @Override
    public Collection<String> listBlogsByCategoryId(Integer categoryId) {
        final String categoryFragmentationNumber = (String) redisTemplate.opsForValue().get(constantConfiguration.CATEGORY_BLOGS_FRAGMENTATION + categoryId);
        List<Object> list = new ArrayList<>();

        // 如果分片数 > 1, 推送12个，8个最新分片中的博客，4个随机分片中的博客
        if (Integer.parseInt(categoryFragmentationNumber) > 1) {
            final List<Object> listFinal = redisTemplate.opsForSet().randomMembers(constantConfiguration.CATEGORY_BLOGS + categoryId + ":" + categoryFragmentationNumber, 8);
            // 获取1 - 分片数的随机数
            final Random random = new Random();
            final int randomFragmentationNumber = random.nextInt(Integer.parseInt(categoryFragmentationNumber) - 1) + 1;
            final List<Object> listRandom = redisTemplate.opsForSet().randomMembers(constantConfiguration.CATEGORY_BLOGS + categoryId + ":" + randomFragmentationNumber, 4);
            list.addAll(listFinal);
            list.addAll(listRandom);
        } else {
            list = redisTemplate.opsForSet().randomMembers(constantConfiguration.CATEGORY_BLOGS + categoryId + ":" + categoryFragmentationNumber, 12);
        }

        // 可能有null
        final HashSet<String> result = new HashSet<>();
        for (Object o : list) {
            if (o != null) {
                result.add(o.toString());
            }
        }

        return result;
    }

    /**
     * 删除分类库中博客
     * @param blog
     */
    @Override
    @Async("asyncExecutor")
    public void deletCategoryStockIn(Blogs blog) {
        final String categoryId = String.valueOf(blog.getCategoryId());
        final String categoryFragmentationNumber = blog.getCategoryFragmentation();//冗余分片数字段

        redisTemplate.opsForSet().remove(constantConfiguration.CATEGORY_BLOGS + categoryId + ":" + categoryFragmentationNumber, String.valueOf(blog.getId()));
    }

    /**
     * 删除标签库中的博客
     * @param blog
     */
    @Override
    @Async("asyncExecutor")
    public void deletTagsStockIn(Blogs blog) {
        final Integer blogId = blog.getId();
        //获取该博客标签列表
        List<String> tags =blogsMapper.getTagsByBlogId(blogId);
        //使用pipelined批量执行
        redisTemplate.executePipelined((RedisCallback<Object>) connection -> {
            for (String tag : tags) {
                connection.sRem((constantConfiguration.TAG_BLOGS + tag).getBytes(), String.valueOf(blogId).getBytes());
            }
            return null;
        });
    }

    /**
     * 构造概率数组，保存的元素是标签 -> probabilityArray = ["标签A", "标签A", "标签A", "标签B", "标签B", "标签C"]
     *
     * @param modelMap
     * @return {@link String[] }
     */
    private String[] initProbabilityArray(Map<Object, Object> modelMap) {
        // key: 标签 value: 概率
        Map<String, Integer> probabilityMap = new HashMap<>();
        int size = modelMap.size();
        final AtomicInteger n = new AtomicInteger(0);

        modelMap.forEach((key, value) -> {
            double probabilityValue = Double.parseDouble(value.toString());
            int probability = (int) ((probabilityValue + size) / size);
            n.getAndAdd(probability);
            probabilityMap.put((String) key, probability);
        });
        final String[] probabilityArray = new String[n.get()];

        final AtomicInteger index = new AtomicInteger(0);
        //初始化数组
        probabilityMap.forEach((tag, probability) -> {
            int i = index.get();
            int limit = i + probability;
            while (i < limit) {
                probabilityArray[i++] = tag;
            }
            index.set(limit);
        });
        return probabilityArray;
    }
}
